@@ -55,7 +55,7 @@ class RangeUpdateVisitor : public AstDefaultVisitor {
 public:
     virtual void visitNode(Ast* node) {
         AstDefaultVisitor::visitNode(node);
-        if ( node && node->parent ) {
+        if ( node && node->parent && node->parent->astType != Ast::AttributeAstType ) {
             if ( ( node->parent->endLine <= node->endLine && node->parent->endCol <= node->endCol )
                  || node->parent->endLine < node->endLine )
             {
@@ -66,6 +66,39 @@ public:
     };
 };
 
+class NextAstFindVisitor : public AstDefaultVisitor {
+public:
+    KTextEditor::Cursor findNext(Python::Ast* node) {
+        m_root = node;
+        auto parent = node;
+        while ( parent && parent->parent && parent->parent->isExpression() ) {
+            parent = parent->parent;
+        }
+        visitNode(parent);
+
+        while ( ! m_next.isValid() && parent->parent ) {
+            // no next expression found in that statement, advance to the next statement
+            parent = parent->parent;
+            visitNode(parent);
+        }
+
+        return m_next;
+    };
+    virtual void visitNode(Python::Ast* node) override {
+        if ( ! node ) {
+            return;
+        }
+        AstDefaultVisitor::visitNode(node);
+        if ( node->start() > m_root->start() && ! node->isChildOf(m_root) ) {
+            m_next = (m_next < node->start() && m_next.isValid()) ? m_next : node->start();
+        }
+    }
+
+private:
+    KTextEditor::Cursor m_next{-1, -1};
+    Ast* m_root;
+};
+
 // This class is used to fix some of the remaining issues
 // with the ranges of objects obtained from the python parser.
 // Issues addressed are:
@@ -79,19 +112,56 @@ public:
 class RangeFixVisitor : public AstDefaultVisitor {
 public:
     RangeFixVisitor(const QString& contents)
-        : lines(contents.split('\n')) { }; // TODO can this be \r\n?
-    virtual void visitFunctionDefinition(FunctionDefinitionAst* node) {
+        : lines(contents.split('\n')) { };
+    virtual void visitFunctionDefinition(FunctionDefinitionAst* node) override {
         cutDefinitionPreamble(node->name, "def");
         AstDefaultVisitor::visitFunctionDefinition(node);
     };
-    virtual void visitClassDefinition(ClassDefinitionAst* node) {
+    virtual void visitClassDefinition(ClassDefinitionAst* node) override {
         cutDefinitionPreamble(node->name, "class");
         AstDefaultVisitor::visitClassDefinition(node);
     };
+
+    virtual void visitAttribute(AttributeAst* node) override {
+        // Work around the weird way to count columns in Python's AST module.
+
+        NextAstFindVisitor v;
+        auto next_start = v.findNext(node);
+        if ( ! next_start.isValid() ) {
+            // use end of document as reference
+            next_start = {lines.size() - 1, lines.last().size() - 1};
+        }
+
+        QString line = lines.at(node->startLine);
+        auto lineno = next_start.line();
+        auto colno = next_start.column();
+        if ( ! (next_start > node->start()) ) {
+            colno = -1;
+            lineno = node->startLine;
+        }
+        // sub-expression
+        do {
+            line = lines.at(lineno);
+            if ( colno != -1 ) {
+                line = line.left(colno);
+            }
+            colno = -1;
+            lineno -= 1;
+        } while ( ! line.contains(node->attribute->value) && lineno >= 0 );
+
+        node->startLine = lineno + 1;
+        node->endLine = node->startLine;
+        node->startCol = line.lastIndexOf(node->attribute->value);
+        node->endCol = node->startCol + node->attribute->value.length() - 1;
+        node->attribute->copyRange(node);
+
+        AstDefaultVisitor::visitAttribute(node);
+    };
+
     // alias for imports (import foo as bar, baz as bang)
     // no strings, brackets, or whatever are allowed here, so the "parser"
     // can be very straightforward.
-    virtual void visitImport(ImportAst* node) {
+    virtual void visitImport(ImportAst* node) override {
         AstDefaultVisitor::visitImport(node);
         int aliasIndex = 0;
         foreach ( AliasAst* alias, node->names ) {
@@ -99,8 +169,9 @@ public:
             aliasIndex += 1;
         }
     };
+
     // alias for exceptions (except FooBarException as somethingterriblehappened: ...)
-    virtual void visitExceptionHandler(ExceptionHandlerAst* node) {
+    virtual void visitExceptionHandler(ExceptionHandlerAst* node) override {
         AstDefaultVisitor::visitExceptionHandler(node);
         if ( ! node->name ) {
             return;
@@ -111,8 +182,11 @@ public:
         node->name->startCol = end - back;
         node->name->endCol = end;
     }
+
 private:
     const QStringList lines;
+    QVector<KTextEditor::Cursor> dots;
+    KTextEditor::Cursor attributeStart;
 
 
     // skip the decorators and the "def" at the beginning
@@ -133,14 +207,16 @@ private:
 
         // cut away decorators
         while ( currentLine < lines.size() ) {
-            if ( ! lines.at(currentLine).trimmed().startsWith('@') ) {
+            if ( lines.at(currentLine).trimmed().startsWith(defKeyword) ) {
                 // it's not a decorator, so stop skipping lines.
                 break;
             }
             currentLine += 1;
         }
+//         qDebug() << "FIX:" << fixNode->range();
         fixNode->startLine = currentLine;
         fixNode->endLine = currentLine;
+//         qDebug() << "FIXED:" << fixNode->range() << fixNode->astType;
 
         // cut away the "def" / "class"
         int currentColumn = -1;
@@ -379,7 +455,7 @@ struct PythonInitializer : private QMutexLocker {
 
 CodeAst::Ptr AstBuilder::parse(KUrl filename, QString &contents)
 {
-    qDebug() << " ====> AST     ====>     building abstract syntax tree for " << filename.path();
+    kDebug() << " ====> AST     ====>     building abstract syntax tree for " << filename.path();
     
     Py_NoSiteFlag = 1;
     
@@ -407,7 +483,7 @@ CodeAst::Ptr AstBuilder::parse(KUrl filename, QString &contents)
     mod_ty syntaxtree = PyParser_ASTFromString(contents.toUtf8().data(), "<kdev-editor-contents>", file_input, &flags, arena);
 
     if ( ! syntaxtree ) {
-        qDebug() << " ====< parse error, trying to fix";
+        kDebug() << " ====< parse error, trying to fix";
         
         PyErr_Fetch(&exception, &value, &backtrace);
         kDebug() << "Error objects: " << exception << value << backtrace;
@@ -415,7 +491,7 @@ CodeAst::Ptr AstBuilder::parse(KUrl filename, QString &contents)
         
         PyObject* errorMessage_str = PyTuple_GetItem(value, 0);
         PyObject* errorDetails_tuple = PyTuple_GetItem(value, 1);
-        qDebug() << "Eventual errors while extracting tuple: ";
+        kDebug() << "Eventual errors while extracting tuple: ";
         PyObject_Print(errorMessage_str, stderr, Py_PRINT_RAW);
        
         if ( ! errorDetails_tuple ) {
