@@ -29,6 +29,7 @@
 #include "checks/controlflowgraphbuilder.h"
 #include "checks/dataaccessvisitor.h"
 #include "kshell.h"
+#include "duchain/helpers.h"
 
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
@@ -45,6 +46,7 @@
 #include <interfaces/ilanguage.h>
 #include <interfaces/foregroundlock.h>
 #include <interfaces/icore.h>
+#include <interfaces/iprojectcontroller.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/idocumentcontroller.h>
 
@@ -87,7 +89,15 @@ void ParseJob::run()
         return abortJob();
     }
     
-    qDebug() << " ====> PARSING ====> parsing file " << document().toUrl() << "; has priority" << parsePriority();
+    kDebug() << " ====> PARSING ====> parsing file " << document().toUrl() << "; has priority" << parsePriority();
+
+    {
+        QMutexLocker l(&Helper::projectPathLock);
+        Helper::projectSearchPaths.clear();
+        foreach  (IProject* project, ICore::self()->projectController()->projects() ) {
+            Helper::projectSearchPaths.append(KUrl(project->folder().url()));
+        }
+    }
     
     // lock the URL so no other parse job can run on this document
     QReadLocker parselock(languageSupport()->language()->parseLock());
@@ -103,7 +113,7 @@ void ParseJob::run()
                 continue;
             }
             if ( ! file->needsUpdate() && file->featuresSatisfied(minimumFeatures()) && file->topContext() ) {
-                qDebug() << " ====> NOOP    ====> Already up to date:" << document().str();
+                kDebug() << " ====> NOOP    ====> Already up to date:" << document().str();
                 setDuChain(file->topContext());
                 if ( ICore::self()->languageController()->backgroundParser()->trackerForUrl(document()) ) {
                     lock.unlock();
